<?php

namespace Picea\ControlStructure;

abstract class AbstractLoop implements ControlStructure {

    public function parse(\Picea\Compiler\Context &$context, ?string $arguments, string $token, array $options = []) : string
    {
        switch($token) {
            case "while":
            case "foreach":
                $name = "$".uniqid("{$token}_");
                
                $stack = array_filter($context->iterationStack ?? [], function($item) {
                    return ! $item['or'];
                });
                
                $count = count($stack);

                if ( $count > 0 ) {
                    $name .= "[" . end($stack)['uid'] . "]";
                }

                $context->iterationStack[] = [
                    'or' => false,
                    'uid' => $name,
                    'token' => "end{$token}",
                ];

                return "<?php $token ($arguments): $name = ( $name ?? 0 ) + 1; ?>";

            case "endwhile":
            case "endforeach":
                $last = end($context->iterationStack);

                if ( $last['or'] === false ) {
                    $output = "<?php $token; ?>";
                }
                else {
                    $output = "<?php endif; if ( isset({$last['uid']}) ) unset({$last['uid']}); ?>";
                }

                array_pop($context->iterationStack);
                
                return $output;
        }
    }
}
